import logging

from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from custom_admin.api.serializers.auth_response import AdminAuthResponseSerializer

logger = logging.getLogger('custom_admin')


class AdminAuthToken(APIView):

    def get_response(self, user, request):
        context = {
            'request': request,
        }
        data = AdminAuthResponseSerializer(user, context=context).data
        return Response(data, status=status.HTTP_200_OK)

    def post(self, request, *args, **kwargs):
        username = request.data.get('username')
        password = request.data.get('password')

        error_response = Response({'non_field_errors': [_('The user is not found or does not have administrator rights.')]}, 401)

        user = get_user_model().objects.filter(username=username).first()
        if not user:
            logger.info('ADMIN BYPASS Login user username:%s not found', username)
            return error_response

        if not user.check_password(password):
            logger.info('ADMIN BYPASS Login user username:%s wrong password', username)
            return error_response

        if not user.is_staff or not user.is_active:
            logger.info('ADMIN BYPASS Login user username:%s not staff or not active', username)
            return error_response

        return self.get_response(user, request)



import logging
import operator
from functools import reduce

from django.apps import apps
from django.db import models
from django.db.models import Manager, Q
from django.db.models.query import QuerySet
from django_filters.utils import get_model_field
from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.views import APIView

from custom_admin.api.permissions import AdminPermission

log = logging.getLogger("admin")

INT_FIELDS = (
    models.PositiveIntegerField,
    models.IntegerField,
    models.AutoField,
    models.BigIntegerField,
)


class AutoCompeteSerializer(serializers.Serializer):
    viewname = serializers.CharField(required=False)
    field_slug = serializers.JSONField(required=False)

    search_string = serializers.CharField(required=False, allow_null=True, allow_blank=True)
    limit = serializers.IntegerField(required=False, allow_null=True)
    existed_choices = serializers.ListField(
        child=serializers.IntegerField(),
        required=False,
        allow_empty=True,
        allow_null=True,
    )


class AutoCompeteView(APIView):
    throttle_classes = []
    permission_classes = (AdminPermission,)

    default_fields = {
        'Permission': ('id', 'codename', 'name'),
        'Group': ('id', 'name'),
        'Country': ('id', 'name', 'alt_names__name'),
        'City': ('id', 'name', 'alt_names__name'),
        'ContentType': ('id', 'model', 'app_label')
    }

    def get_search_filter(self, model, search_string: str):
        filters = []

        autocomplite_fields = getattr(
            model, 'admin_autocomplite_fields',
            self.default_fields.get(model.__name__, ('id',))
        )

        for field_name in autocomplite_fields:
            field = get_model_field(model, field_name)
            value = search_string

            filter_lookup = None
            if issubclass(field.__class__, INT_FIELDS):
                # Int field
                try:
                    value = int(search_string)
                    filter_lookup = f'{field_name}__exact'
                except ValueError:
                    continue
            else:
                # String field
                filter_lookup = f'{field_name}__icontains'

            if filter_lookup:
                field_filter = Q(**{filter_lookup: value})
                filters.append(field_filter)

        if not filters:
            return None

        return reduce(operator.or_, filters)

    def get_queryset_by_view(self, model, viewname, form_data, field_slug):

        if not viewname:
            return model.objects.all()

        from custom_admin.utils.register_admin_viewsets import _ADMIN_VIEWS
        view = _ADMIN_VIEWS[viewname]
        serializer = view.get_serializer_class()()

        field = serializer.fields.get(field_slug)
        if not field:
            raise serializers.ValidationError(f'Model {self.app_label}.{self.model_name} view "{viewname}" field slug "{field_slug}" is not found')

        if isinstance(field, serializers.ManyRelatedField):
            field = field.child_relation

        filter_queryset = getattr(field, 'filter_queryset', None)
        qs = getattr(field, 'queryset')

        if form_data is None:
            form_data = {}

        if filter_queryset:
            filtered_qs = filter_queryset(qs, form_data, self.request)
            log.debug(
                'AUTOCOMPLETE view:%s using:%s init form_data.keys:%s result:%s',
                viewname, filter_queryset.__name__, form_data.keys(), filtered_qs,
            )
            return filtered_qs

        if not isinstance(qs, (QuerySet, Manager)):
            raise serializers.ValidationError(f'Model {self.app_label}.{self.model_name} view "{viewname}" autocomplete for {field_slug}')

        return qs.all()

    @staticmethod
    def _get_str_value(obj):
        if not obj:
            return

        try:
            return str(obj) or str(obj.id)
        except TypeError as e:
            log.error('Admin autocomplete: error get __str__ for %s', obj)
            return str(obj.id)

    def post(self, request, model_name, app_label, *args, **kwargs):
        self.app_label = app_label
        self.model_name = model_name

        serializer = AutoCompeteSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        search_string = serializer.validated_data.get('search_string')
        limit = serializer.validated_data.get('limit')

        model = apps.get_model(app_label=app_label, model_name=model_name)
        if not model:
            raise serializers.ValidationError(f'Model {self.app_label}.{self.model_name} not found')

        view_queryset = self.get_queryset_by_view(
            model,
            viewname=serializer.validated_data.get('viewname'),
            form_data=serializer.validated_data.get('form_data'),
            field_slug=serializer.validated_data.get('field_slug'),
        )

        qs = view_queryset
        if search_string:
            filters = self.get_search_filter(model, search_string)
            if not filters:
                return Response([])
            qs = qs.filter(filters).order_by('id').distinct('id')

        if limit:
            qs = qs[:limit]

        existed_choices = serializer.validated_data.get('existed_choices', [])
        if existed_choices:

            # Добавляет в результат уже выбранные варианты
            filter_ids = view_queryset.filter(id__in=existed_choices)
            if qs:
                qs = qs.union(filter_ids)
            else:
                qs = filter_ids

        formated_values = [
            {'id': obj.id, 'text': self._get_str_value(obj)}
            for obj in qs
        ]
        return Response(formated_values)



from custom_admin.controllers.schema_generator import ViewSetSchemaGenerator


def get_schema(router, request) -> dict:
    sections = {}

    for _prefix, viewset, basename in router.registry:

        has_perm = True
        for perm in viewset.permission_classes:
            if not perm().has_permission(request, viewset):
                has_perm = False
                break

        if has_perm:
            generator = ViewSetSchemaGenerator(viewset, router, basename, request)
            sections[basename] = generator.get_viewset_urls()

    return sections



import logging

from django.conf import settings
from django.utils import translation
from rest_framework.response import Response
from rest_framework.views import APIView

from custom_admin.api.permissions import AdminPermission
from custom_admin.utils import get_schema

log = logging.getLogger("request")


class GetAdminSectionsAPIView(APIView):
    '''
    Get all scheme for admin frontend
    '''
    throttle_classes = []
    permission_classes = (AdminPermission, )
    router = None

    def get(self, request, *args, **kwargs):
        lang = request.headers.get('Accept-Language') or getattr(settings, 'LANGUAGE_CODE', None) or 'en'
        translation.activate(lang)
        response = Response({
            'sections': get_schema(self.router, request),
            'language': lang,
            'languages': {lang[0]: lang[1] for lang in settings.LANGUAGES},
        })
        return response



import logging
import typing
from datetime import datetime

from django.conf import settings
from django.utils.decorators import classonlymethod
from rest_framework.response import Response

from custom_admin.controllers import AdminLogManager
from custom_admin.api.inlines.interfaces import InlineResultInterface
from custom_admin.controllers.filters_schema import get_filters_class_data

log = logging.getLogger("admin-actions")


class InlineError(Exception):
    pass


class ViewActionsInlineMixIn:
    """
    Handler for action with type='Inline'
    from rest_framework.decorators import action

    Action decorator must contain
    - inline_type: InlinesType
    - filterset_class: instance of BaseAdminFilterSet
    - actions: list of funcs
    """

    def get_filters(self, filterset_class, request):
        if filterset_class is None:
            return {}, []

        filterset_fields = get_filters_class_data(None, filterset_class, request)
        filters = {}
        for field_slug, field_info in filterset_fields.items():
            if field_info.get('type') in ('ManyRelatedField', 'AdminManyRelatedField'):
                filters[field_slug] = request.GET.getlist(field_slug, [])

            elif field_info.get('type') == 'DateFromToRangeFilter':
                before = request.GET.get(f'{field_slug}_before')
                after = request.GET.get(f'{field_slug}_after')

                filters[f'{field_slug}_before'] = datetime.strptime(before, "%Y-%m-%d") if before else None
                filters[f'{field_slug}_after'] = datetime.strptime(after, "%Y-%m-%d") if after else None

            else:
                filters[field_slug] = request.GET.get(field_slug)

        return filters

    @classonlymethod
    def as_view(cls, *args, **initkwargs):
        # parameters of rest_framework.decorators.action
        initkwargs.pop('inline_type', None)
        initkwargs.pop('filterset_class', None)
        initkwargs.pop('actions', None)
        return super().as_view(*args, **initkwargs)

    def perform_action(self, view, request, handler_result, inline_action, actions):
        for action in actions:
            if action.__name__ == inline_action:
                action_result = action(view, request, handler_result)
                AdminLogManager(self.request.user).register_action(
                    action.__name__, action.short_description, self, request
                )
                return action_result

        return Response(f'Action {action.__name__} not found', status=400)

    def handle_inline(self, view_handler, request, handler_kwargs, *args, **kwargs) -> Response:
        # inline_type = handler_kwargs.get('inline_type')

        filterset_class = handler_kwargs.get('filterset_class')

        permission_classes = handler_kwargs.get('permission_classes')
        for permission_class in permission_classes:
            if not permission_class().has_permission(request, None):
                return Response('You don\'t have access', status=401)

        filters = self.get_filters(filterset_class, request)

        page = request.GET.get('page', 1)
        page_size = request.GET.get('limit', settings.REST_FRAMEWORK['PAGE_SIZE'])

        inline_action = request.GET.get('inline_action')

        pk = kwargs.pop('pk', None)

        try:
            handler_result: typing.Optional[InlineResultInterface] = view_handler(request, pk, filters, page, page_size, inline_action, *args, **kwargs)
        except Exception as e:
            log.exception('Action error:"%s"', str(e))
            raise InlineError(f'Action error: {e}') from e

        # Inlines can have actions as export_csv_inline
        if inline_action:
            return self.perform_action(view_handler, request, handler_result, inline_action, handler_kwargs.get('actions'))

        if handler_result is None:
            return Response()

        if not issubclass(handler_result.__class__, InlineResultInterface):
            raise InlineError(f'Action view "{view_handler.__name__}" must InlineResultInterface or None; result:{handler_result}')

        return Response(handler_result.asdict())



import json
import typing
from urllib.parse import unquote

from django.http import QueryDict
from django_filters import rest_framework
from rest_framework import filters

from custom_admin.api.filters.base_admin_filter import BaseAdminFilterSet


def get_filter_info(request) -> typing.Optional[dict]:
    if not request.GET or 'filter_info' not in request.GET:
        return

    filter_info = request.GET['filter_info']
    if not filter_info:
        return

    return json.loads(unquote(filter_info))


class CustomSearchFilter(filters.SearchFilter):
    def get_search_terms(self, request):
        search = super().get_search_terms(request)

        filter_info = get_filter_info(request)
        if filter_info:
            search = filter_info.get('search')
            if search:
                return search

        return search


class CustomOrderingFilter(filters.OrderingFilter):
    def get_ordering(self, request, queryset, view):
        ordering = super().get_ordering(request, queryset, view)

        filter_info = get_filter_info(request)
        if filter_info:
            ordering = filter_info.get('ordering')
            if ordering:
                return ordering

        return ordering


class CustomFilterBackend(rest_framework.DjangoFilterBackend):
    filterset_base = BaseAdminFilterSet

    def get_filterset_kwargs(self, request, queryset, view):
        data = request.query_params

        filter_info = get_filter_info(request)
        if filter_info:
            _filters = filter_info.get('filters')
            if _filters:
                data = _filters

        return {
            'data': data,
            'queryset': queryset,
            'request': request,
        }



from rest_framework import serializers
from rest_framework.authtoken.models import Token

from django.contrib.auth import get_user_model

from custom_admin.api.serializers.base_serializer import AdminModelSerializer


class AdminAuthResponseSerializer(AdminModelSerializer):
    token = serializers.SerializerMethodField()
    language = serializers.SerializerMethodField()

    class Meta:
        model = get_user_model()
        fields = [
            'pk',
            'username',
            'token',
            'language',
        ]

    def get_token(self, user):
        token, _ = Token.objects.get_or_create(user=user)
        return token.key

    def get_language(self, user):
        return getattr(user, 'language', 'en')



import logging

from rest_framework import relations, serializers

from django.db import models

from custom_admin.api import fields
from custom_admin.controllers import AdminLogManager

log = logging.getLogger("custom_admin")


class AdminFieldsMixin:
    serializer_related_field = fields.AdminPrimaryKeyRelatedField
    serializer_related_to_field = relations.SlugRelatedField
    serializer_choice_field = fields.AdminChoiceField

    serializer_field_mapping = serializers.ModelSerializer.serializer_field_mapping
    serializer_field_mapping[models.FileField] = fields.Base64FileField
    serializer_field_mapping[models.ImageField] = fields.Base64ImageField
    serializer_field_mapping[models.CharField] = fields.AdminCharField
    serializer_field_mapping[models.TextField] = fields.AdminCharField
    serializer_field_mapping[models.JSONField] = fields.AdminJsonField

    serializer_field_mapping[models.PositiveIntegerField] = fields.PositiveIntegerField


class AdminSerializer(AdminFieldsMixin, serializers.Serializer):
    pass


class AdminModelSerializer(AdminFieldsMixin, serializers.ModelSerializer):

    def create(self, validated_data):
        obj = super().create(validated_data)

        AdminLogManager(self.context['request'].user).register_create(self, validated_data, obj)
        return obj

    def update(self, instance, validated_data):
        data_before_update = {
            k: AdminLogManager.get_field_value(instance, k)
            for k, v in validated_data.items()
        }
        obj = super().update(instance, validated_data)

        AdminLogManager(self.context['request'].user).register_update(self, data_before_update, validated_data, obj)
        return obj


class TranslatedModelSerializer(AdminModelSerializer):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        from modeltranslation.translator import translator
        self.trans_opts = translator.get_options_for_model(self.Meta.model)

    def get_field_names(self, declared_fields, info):
        from modeltranslation.utils import get_translation_fields
        result = super().get_field_names(declared_fields, info)
        for f in self.trans_opts.fields:
            trans_fields = get_translation_fields(f)
            result += tuple(trans_fields)

        return result

    def get_extra_kwargs(self):
        from modeltranslation.utils import get_translation_fields
        extra_kwargs = super().get_extra_kwargs()
        for field in self.trans_opts.fields:

            if field not in extra_kwargs:
                extra_kwargs[field] = {}

            extra_kwargs[field]['required'] = False
            extra_kwargs[field]['allow_null'] = True
            extra_kwargs[field]['allow_blank'] = True

            if field in extra_kwargs:
                for tanslated_field in get_translation_fields(field):
                    extra_kwargs[tanslated_field] = extra_kwargs.get(field)

        return extra_kwargs


class OrderedAdminModelSerializer(AdminModelSerializer):
    """
    Класс для моделей с приоритетом OrderedModel

    если поле в моделе отличается от order, в сериализаторе надо его указать в order_field
    """
    order_field = 'order'

    def get_order_field(self):
        model_class = self.Meta.model
        order_field_name = getattr(model_class, "order_field_name", self.order_field)

        if not order_field_name:
            raise AttributeError(
                "The `order_field_name` attribute must be set to use the "
                "OrderedModelSerializer. Either inherit from OrderedModel "
                "(to use the default `order` field) or inherit from "
                "`OrderedModelBase` and set the `order_field_name` attribute "
                "on the " + model_class.__name__ + " Model class."
            )

        return order_field_name

    def get_fields(self):
        # make sure that DRF considers the ordering field writable
        order_field = self.get_order_field()
        d = super().get_fields()
        for name, field in d.items():
            if name == order_field:
                if field.read_only:
                    d[name] = fields.IntegerField()
        return d

    def update(self, instance, validated_data):
        """
        Update the instance.
        If the `order_field_name` attribute is passed in the validated data,
        the instance will be moved to the specified order.
        Returns:
            Model: The updated instance.
        """

        order = None
        order_field = self.get_order_field()

        if order_field in validated_data:
            order = validated_data.pop(order_field)

        instance = super().update(instance, validated_data)

        if order is not None:
            instance.to(order)

        return instance

    def create(self, validated_data):
        """
        Create a new instance.
        If the `order_field_name` attribute is passed in the validated data,
        the instance will be created at the specified order.
        Returns:
            Model: The created instance.
        """
        order = None
        order_field = self.get_order_field()

        if order_field in validated_data:
            order = validated_data.pop(order_field)

        instance = super().create(validated_data)

        if order is not None:
            instance.to(order)

        return instance



from django_filters import rest_framework as drf_filters


class ChoiceFilter(drf_filters.ChoiceFilter):

    def __init__(self, *args, **kwargs):
        if callable(kwargs['choices']):
            kwargs['choices'] = kwargs['choices']()

        super().__init__(*args, **kwargs)



from django.utils.functional import lazy
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from custom_admin.api import filters as admin_filters
from custom_admin.api.serializers.base_serializer import AdminModelSerializer
from custom_admin.api.views.base_admin_viewset import WithoutCreateBaseAdminViewSet
from custom_admin.models import AdminLog


def get_views_choices():
    from custom_admin.utils.register_admin_viewsets import _ADMIN_VIEWS
    return [
        (viewname, f'{view._info.title} - {view.get_view_title()}')
        for viewname, view in _ADMIN_VIEWS.items()
    ]


class AdminLogFilterSet(admin_filters.BaseAdminFilterSet):
    section = admin_filters.ChoiceFilter(
        label=_('Раздел'), choices=lazy(get_views_choices, tuple),
    )

    class Meta:
        model = AdminLog
        fields = (
            'section',
            'staff',
            'action_type',
            'created_at',
        )


class AdminLogAdminSerializer(AdminModelSerializer):
    section = serializers.SerializerMethodField(label=_('Раздел'))

    class Meta:
        model = AdminLog
        fields = '__all__'
        read_only_fields = (
            'content',
            'created_at',
            'section',
            'action_type',
            'staff',
            'title',
        )

    def get_section(self, obj):
        from custom_admin.utils.register_admin_viewsets import _ADMIN_VIEWS
        view = _ADMIN_VIEWS.get(obj.section)
        if not view:
            return obj.section

        return f'{view._info.title} - {view.get_view_title()}'


class AdminLogAdminViewSet(WithoutCreateBaseAdminViewSet):
    queryset = AdminLog.objects.all()
    serializer_class = AdminLogAdminSerializer
    search_fields = ['id', 'staff__username', 'title']
    filterset_class = AdminLogFilterSet
    list_display = [
        'id',
        'staff',
        'action_type',
        'section',
        'title',
        'created_at',
    ]
    ordering_fields = ['id', 'staff', 'section', 'action_type', 'title']



import datetime

from django.contrib.auth import get_user_model
from django.test.utils import override_settings
from django.urls import reverse
from rest_framework.test import APITestCase

from custom_admin.api.tests.urls import ADMIN_URLS
from custom_admin.models.admin_log import AdminLog, AdminLogType

User = get_user_model()

ATTRS = '?page=1&limit=25&filter_info=%257B%2522search%2522%253A%2522usern%2522%252C%2522filters%2522%253A%257B%2522section%2522%253A%2522note%2522%252C%2522staff%2522%253A%255B2%255D%252C%2522action_type%2522%253A%2522create%2522%252C%2522created_at%2522%253A%257B%2522from%2522%253A%25222024-05-08T01%253A48%253A00%2522%252C%2522to%2522%253A%25222024-05-23T01%253A48%253A00%2522%257D%257D%257D'


@override_settings(ADMIN_URLS=ADMIN_URLS)
class FiltersTestAPI(APITestCase):

    def setUp(self) -> None:
        self.admin = User.objects.create(
            id=2,
            username="username",
            is_staff=True,
            is_superuser=True,
        )
        self.other_admin = User.objects.create(
            id=3,
            username="other",
            is_staff=True,
            is_superuser=True,
        )

        AdminLog.objects.create(
            staff=self.admin,
            section='note',
            action_type=AdminLogType.CREATE,
            created_at=datetime.datetime(2024, 5, 10),
        )
        AdminLog.objects.create(
            staff=self.admin,
            section='note',
            action_type=AdminLogType.CREATE,
            created_at=datetime.datetime(2021, 5, 5),
        )
        AdminLog.objects.create(
            staff=self.other_admin,
            section='note',
            action_type=AdminLogType.CREATE,
            created_at=datetime.datetime(2021, 11, 16),
        )
        AdminLog.objects.create(
            staff=self.other_admin,
            section='section',
            action_type=AdminLogType.ACTION,
            created_at=datetime.datetime(2021, 11, 16),
        )

    def test_api_filter(self):
        self.client.force_authenticate(user=self.admin)
        url = reverse('adminlog-list')
        response = self.client.get(f'{url}?{ATTRS}')
        self.assertEqual(response.status_code, 200, response.content.decode())

        data = response.json()
        self.assertEqual(data['count'], 1)



import logging

from django.contrib.auth import get_user_model
from django.db.models.fields.files import ImageFieldFile

from custom_admin.models import AdminLog
from custom_admin.models.admin_log import AdminLogType

log = logging.getLogger("custom_admin")


class AdminLogManager:
    staff: get_user_model()

    def __init__(self, staff):
        self.staff = staff

    def register_create(self, serializer, validated_data, created_instance) -> AdminLog:
        content = {
            k: str(v)
            for k, v in validated_data.items()
        }

        admin_log = AdminLog.objects.create(
            staff=self.staff,
            section=serializer.context['viewname'],
            action_type=AdminLogType.CREATE,
            content=content,
            title=str(created_instance)[:63],
        )
        log.info(
            'ADMIN %s #%s CREATED "%s" #%s viewname:%s log:%s',
            self.staff.username, self.staff.id,
            created_instance._meta.verbose_name, created_instance.id,
            serializer.context['viewname'], admin_log.id,
            extra={'validated_data': validated_data}
        )

    @staticmethod
    def get_image_safe(image):
        if image and hasattr(image, 'url'):
            return image.url
        return None

    def check_equal(self, original_value, new_value):
        if isinstance(original_value, ImageFieldFile) and isinstance(new_value, ImageFieldFile):
            return self.get_image_safe(original_value) != self.get_image_safe(new_value)
        else:
            return original_value != new_value

    @staticmethod
    def get_field_value(instance, k):
        value = getattr(instance, k, None)
        if value.__class__.__name__ == 'ManyRelatedManager':
            return list(value.all().values_list('id', flat=True))

        return value

    def register_update(self, serializer, data_before_update, validated_data, updated_instance) -> AdminLog:
        content = {}
        for k in validated_data.keys():
            original_value = data_before_update.get(k)
            new_value = self.get_field_value(updated_instance, k)

            if self.check_equal(original_value, new_value):
                content[k] = {'from': str(original_value), 'to': str(new_value)}

        if not content:
            return

        admin_log = AdminLog.objects.create(
            staff=self.staff,
            section=serializer.context['viewname'],
            action_type=AdminLogType.EDIT,
            content=content,
            title=str(updated_instance)[:63],
        )
        log.info(
            'ADMIN %s #%s UPDATE "%s" #%s viewname:%s log:%s',
            self.staff.username, self.staff.id,
            updated_instance._meta.verbose_name, updated_instance.id,
            serializer.context['viewname'], admin_log.id,
            extra={'validated_data': validated_data}
        )
        return admin_log

    def register_action(self, action_name, short_description, view, request) -> AdminLog:
        admin_log = AdminLog.objects.create(
            staff=self.staff,
            section=view.get_view_viewname(),
            action_type=AdminLogType.ACTION,
            content=request.data,
            title=short_description,
        )
        log.info(
            'ADMIN %s #%s ACTION "%s" admin_log:%s short_description:%s',
            self.staff.username, self.staff.id, action_name, admin_log.section, short_description,
        )
        return admin_log



from django.db.models.deletion import ProtectedError
from django.utils.translation import gettext as _

from custom_admin.api.action_functions import admin_action


@admin_action(
    short_description=_("Удалить"),
    icon='mdi-delete-forever',
    base_color='#ff3333',
    variant='outlined',
    confirmation_text=_('Вы уверены, что хотите удалить выбранные записи?'),
)
def delete_action(view, request, queryset, *args, **kwargs):
    try:
        queryset.delete()
    except ProtectedError as e:
        return _(
            'Невозможно удалить некоторые экземпляры модели, '
            'поскольку на них ссылаются через защищенные внешние ключи: %(objects)s') % {
                   'objects': ", ".join([str(obj) for obj in e.protected_objects])
               }, 400

    return _('Записи успешно удалены!'), 200



import typing
from dataclasses import dataclass

from django.utils.module_loading import import_string


@dataclass
class AdminViewSetInfo:
    group_slug: str
    title: str

    views: typing.List[str]

    # https://pictogrammers.com/library/mdi/
    icon: typing.Optional[str] = None

    def __post_init__(self):
        assert len(self.views) > 0, 'AdminViewSetInfo.views is empty'

    def iterate_views(self):
        from custom_admin.api.views import BaseAdminDataViewSet, BaseAdminViewSet

        for view_class_path in self.views:
            view = import_string(view_class_path)
            assert issubclass(view, (BaseAdminDataViewSet, BaseAdminViewSet)), f'Admin viewset "{view_class_path}" {view.__name__} must be instance of BaseAdminDataViewSet or BaseAdminViewSet'
            yield view

    def asdict(self):
        return {
            'group': self.group_slug,
            'group_title': self.title,
            'group_icon': self.icon,
        }



from rest_framework import serializers


class AdminBaseFieldMixin:
    update_only = None
    create_only = None
    list_only = None

    def __init__(self, **kwargs):
        self.update_only = kwargs.pop('update_only', False)
        self.create_only = kwargs.pop('create_only', False)
        self.list_only = kwargs.pop('list_only', False)
        super().__init__(**kwargs)



import typing
from dataclasses import asdict, dataclass, field

from django.core.paginator import Paginator
from rest_framework import serializers

from custom_admin.api.inlines.interfaces import InlineResultInterface
from custom_admin.api.inlines.view_inline_mixin import InlineError


@dataclass
class ColumnInfo:
    width: typing.Optional[int] = None
    fixed: bool = False
    html: bool = False


@InlineResultInterface.register
@dataclass
class InlineTableResult:
    columns_info: typing.Dict[str, ColumnInfo] = field(default_factory=dict)

    messages: typing.List[dict] = field(default_factory=dict)
    paginator: typing.Optional[Paginator] = None

    serializer: typing.Optional[serializers.Serializer] = None

    # Depricated!
    columns: typing.List[str] = field(default_factory=list)
    # Depricated!
    data: typing.List[list] = field(default_factory=list)

    def set_up_serializer(self):
        if not self.serializer:
            return

        self.columns = []

        if hasattr(self.serializer, 'child'):
            self.data = [d.values() for d in self.serializer.data]
            for field_key, field_value in self.serializer.child.get_fields().items():
                self.columns.append(field_value.label)
        else:
            raise InlineError(f'{self.serializer.__class__} is not implemented for InlineTableResult')

    def __post_init__(self):
        self.set_up_serializer()

    def asdict(self) -> dict:
        result = {}

        formated_data = []
        if self.data:
            for line in self.data:
                formated_data.append({
                    str(self.columns[i]): item
                    for i, item in enumerate(line)
                })
            result['data'] = formated_data

        result['columns'] = self.columns
        result['messages'] = self.messages

        result['columns_info'] = {
            k: asdict(v)
            for k, v in self.columns_info.items()
        }

        if self.paginator:
            result['count'] = self.paginator.count
            result['num_pages'] = self.paginator.num_pages

        return result



from datetime import datetime, time

from django_filters import filters
from django_filters.fields import forms

FORMAT = '%Y-%m-%dT%H:%M:%S'


class AsIsField(forms.Field):
    def to_python(self, value):
        return value


class AdminDateFromToRangeFilter(filters.DateFromToRangeFilter):
    field_class = AsIsField

    def filter(self, qs, value):
        if value:
            _from = datetime.combine(datetime.strptime(value['from'], FORMAT), time.min)
            to = datetime.combine(datetime.strptime(value['to'], FORMAT), time.max)
            return qs.filter(**{
                f'{self.field_name}__gte': _from,
                f'{self.field_name}__lte': to,
            })
        return super().filter(qs, value)



from django.core.management.base import BaseCommand

VIEWSET_INFO = '''- {title: <35} {view_slug: <25} model:{model}'''


class Command(BaseCommand):
    def handle(self, *args, **options):
        from custom_admin.utils.register_admin_viewsets import _ADMIN_VIEWS

        if not _ADMIN_VIEWS:
            self.stdout.write(
                self.style.ERROR('settings.ADMIN_URLS is empty')
            )

        groups = {}
        for viewset in _ADMIN_VIEWS.values():
            groups[viewset._info.group_slug] = viewset._info.title

        for group_slug, group_title in groups.items():
            self.stdout.write(f'\n"{group_title}" {group_slug}')

            for view_slug, viewset in _ADMIN_VIEWS.items():
                if viewset._info.group_slug != group_slug:
                    continue

                model = None
                if hasattr(viewset, 'get_model'):
                    model = viewset.get_model()

                if model:
                    model = model.__name__

                line = VIEWSET_INFO.format(
                    view_slug=view_slug,
                    model=model,
                    title=str(viewset.get_view_title()),
                )
                self.stdout.write(
                    self.style.SUCCESS(line)
                )



from django.contrib.auth import get_user_model
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class AdminLogType(models.TextChoices):
    CREATE = 'create', _('Создание')
    EDIT = 'edit', _('Редактирование')
    ACTION = 'action', _('Действие')


class AdminLog(models.Model):
    staff = models.ForeignKey(
        get_user_model(),
        verbose_name=_('Персонал'),
        on_delete=models.PROTECT,
        related_name='admin_actions',
    )
    section = models.CharField(
        _('Раздел'),
        max_length=64,
    )
    title = models.CharField(
        _('Заголовок'),
        max_length=64,
        blank=True, null=True,
    )
    action_type = models.CharField(
        _('Тип действия'),
        max_length=12,
        choices=AdminLogType.choices,
    )

    content = models.JSONField(
        verbose_name=_("Содержимое действия"), default=dict
    )
    created_at = models.DateTimeField(
        verbose_name=_('Время совершения действия'), default=timezone.now,
    )

    class Meta:
        verbose_name = _('Лог действия администратора')
        verbose_name_plural = _('Лог действий администраторов')

    def __str__(self):
        return f'{self.get_action_type_display()} #{self.id} {self.staff.username}'



from .autocomplete import AutoCompeteView
from .base_admin_viewset import BaseAdminDataViewSet
from .base_admin_viewset import BaseAdminViewSet
from .base_admin_viewset import WithoutCreateBaseAdminViewSet
from .base_admin_viewset import WithoutUpdateBaseAdminViewSet
from .base_admin_viewset import ReadOnlyBaseAdminViewSet
from .get_sections import GetAdminSectionsAPIView
from .token_auth import AdminAuthToken



from .view_inline_mixin import ViewActionsInlineMixIn
from .export_csv_inline import export_csv_inline
from .inline_table import InlineTableResult
from .inline_table import ColumnInfo
from .inlines_type import InlinesType
from .inline_graph import InlineGraphResult
from .inline_graph import GraphDataset



from django.utils.translation import gettext as _
from django_filters import filters
from django_filters.utils import get_model_field


class ModelFieldException(Exception):
    pass


def get_filters_fields_data(model, filters):
    result = {}
    for f in filters:
        filter_name = None
        filter_field = None

        if isinstance(f, str):
            filter_name = f
            filter_field = get_model_field(model, f)
            if not filter_field:
                raise ModelFieldException(f'Field "{f}" not found in {model.__name__}')

        elif isinstance(f, (list, tuple)):
            filter_name = f[0]
            filter_field = f[1]

        result[filter_name] = {'type': filter_field.__class__.__name__}

        if result[filter_name]['type'] == 'ManyToManyRel':
            result[filter_name]['label'] = str(_(filter_field.related_model._meta.verbose_name))
        else:
            result[filter_name]['label'] = str(_(filter_field.verbose_name))

        if result[filter_name]['type'] in ('ForeignKey', 'ManyToManyField', 'OneToOneField', 'ManyToManyRel'):
            result[filter_name]['model_name'] = filter_field.related_model.__name__
            result[filter_name]['app_label'] = filter_field.related_model._meta.app_label

        if hasattr(filter_field, 'choices') and filter_field.choices:
            choices = tuple({
                'value': c[0],
                'display_name': str(_(c[1]))
            } for c in filter_field.choices)
            if choices:
                result[filter_name]['choices'] = choices

    return result


def get_filters_class_data(model, filter_class, request) -> dict:
    result = {}

    for filter_name, filter_field in filter_class.get_filters().items():
        name = filter_field.label
        if not name:
            field = get_model_field(model, filter_field.field_name)
            if field:
                name = _(field.verbose_name)

        result[filter_name] = {
            'type': filter_field.__class__.__name__,
            'label': str(name),
        }

        filter_model = None

        if result[filter_name]['type'] in ('ModelChoiceFilter', 'OneToOneField', 'ModelMultipleChoiceFilter'):
            filter_model = filter_field.get_queryset(request).model

        elif result[filter_name]['type'] in ('ManyRelatedField', 'AdminManyRelatedField'):
            filter_model = filter_field.child_relation.queryset.model

        if filter_model:
            model_name = filter_model.__name__
            app_label = filter_model._meta.app_label
            result[filter_name]['model_name'] = model_name
            result[filter_name]['app_label'] = app_label

        if isinstance(filter_field, filters.Filter):
            choices = filter_field.extra.get('choices')
            if choices:
                choices = tuple({
                    'value': c[0],
                    'display_name': str(_(c[1]))
                } for c in choices)
                if choices:
                    result[filter_name]['choices'] = choices

    return result



import logging

from django.contrib.contenttypes.models import ContentType
from rest_framework.permissions import BasePermission

logger = logging.getLogger('custom_admin')


class AdminPermission(BasePermission):
    def has_permission(self, request, view):
        user = request.user

        if not user.is_staff and not user.is_superuser:
            logger.info('ADMIN BYPASS User #%s: User is not saff or superuser', user.id)
            return False

        if not user.is_superuser and view and hasattr(view, 'get_model'):
            model = view.get_model()
            if model is not None:
                content_type = ContentType.objects.get_for_model(model)
                has_perm = user.user_permissions.filter(
                    content_type=content_type
                ).exists()
                if not has_perm:
                    return False

        return True



from django.conf import settings
from rest_framework import routers

from custom_admin.api.viewset_info import AdminViewSetInfo

_ADMIN_VIEWS = {}


def register_admin_viewsets(admin_router: routers.SimpleRouter):
    group: AdminViewSetInfo
    for group in settings.ADMIN_URLS:

        for view in group.iterate_views():
            viewname = view.get_view_viewname()

            _ADMIN_VIEWS[viewname] = view
            view._info = group

            admin_router.register(
                prefix=viewname,
                viewset=view,
                basename=viewname
            )



from .base_serializer import AdminModelSerializer
from .base_serializer import AdminSerializer
from .base_serializer import OrderedAdminModelSerializer
from .base_serializer import TranslatedModelSerializer



import typing

from django.urls import include, path

from custom_admin.api.viewset_info import AdminViewSetInfo

ADMIN_URLS: typing.List[AdminViewSetInfo] = [
    AdminViewSetInfo(
        group_slug='test',
        title='test',
        views=[
            'custom_admin.api.views.defaults.GroupAdminViewSet',
            'custom_admin.api.views.defaults.PermissionsAdminViewSet',
            'custom_admin.api.views.defaults.AdminLogAdminViewSet',
        ]
    ),
]

urlpatterns = [
    path('custom_admin/', include('custom_admin.api.urls')),
]
